==> kite-1.0b1.tar.gz: kite-1.0b1/vm/kite_compiler.c <==
/*****************************************************************************
 * Copyright (c) 2007, Mooneer Salem
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Kite Language organization nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MOONEER SALEM ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MOONEER SALEM BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "objs/kite_object.h"
#include "kite_vm.h"

typedef void* yyscan_t;
int yylex_init(yyscan_t*);
void yyset_extra(void *, yyscan_t);
void yyset_in(FILE *, yyscan_t);
int yylex_destroy(yyscan_t);
int yyparse (void *);

/*****************************************************************************
 * Compile Kite code from a file.
 ****************************************************************************/
kite_object_t *kite_vm_compile_from_file(kite_thread_t *thd, char *file)
{
    FILE *fp;
    kite_object_t *ret;
    
    fp = fopen(file, "r");
    if (fp == NULL) {
        return NULL;
    }

    ret = kite_vm_compile_from_fp(thd, fp, file);
    fclose(fp);
    return ret;
}

/*****************************************************************************
 * Compile Kite code from an open file.
 ****************************************************************************/
kite_object_t *kite_vm_compile_from_fp(kite_thread_t *thd, FILE *fp, char *f) 
{
    yyscan_t scanner;
    kite_compiler_t *compiler;
    kite_object_t *ret;
    
    compiler = calloc(sizeof(kite_compiler_t), 1);
    if (compiler == NULL) {
        return NULL;
    }
    
    compiler->thd = thd;
    compiler->currentLine = 1;
    compiler->file = f;
    kite_push_stack(&compiler->obj_created, FALSE, kite_new_class(thd, NULL, ""));
    
    yylex_init(&scanner);
	yyset_extra(compiler, scanner);
    yyset_in(fp, scanner);
    yyparse(scanner);
	yylex_destroy(scanner);
	
    ret = kite_pop_stack(&compiler->obj_created);
    ret->builtin_data.funcvalue.functype = FUNC_BYTECODE;
    free(compiler);
    return ret;
}

typedef void* YY_BUFFER_STATE;
void yy_switch_to_buffer ( YY_BUFFER_STATE, yyscan_t );
void yy_delete_buffer ( YY_BUFFER_STATE, yyscan_t );
YY_BUFFER_STATE yy_scan_string ( const char *, yyscan_t );

/*****************************************************************************
 * Compile Kite code from a string.
 ****************************************************************************/
kite_object_t *kite_vm_compile_from_string(kite_thread_t *thd, char *code)
{
    kite_object_t *ret;
    yyscan_t scanner;
    kite_compiler_t *compiler;
    YY_BUFFER_STATE bufstate = NULL;
    
    compiler = calloc(sizeof(kite_compiler_t), 1);
    if (compiler == NULL) {
        return NULL;
    }

    compiler->thd = thd;
    compiler->currentLine = 1;
    compiler->file = "?";
    kite_push_stack(&compiler->obj_created, FALSE, kite_new_class(thd, NULL, ""));

    yylex_init(&scanner);
	yyset_extra(compiler, scanner);
    bufstate = yy_scan_string(code, scanner);
    yy_switch_to_buffer(bufstate, scanner);
    yyparse(scanner);
    yy_delete_buffer(bufstate, scanner);
	yylex_destroy(scanner);

    ret = kite_pop_stack(&compiler->obj_created);
    ret->builtin_data.funcvalue.functype = FUNC_BYTECODE;
    free(compiler);
    return ret;
}

==> kite_1.0.2.orig.tar.gz: kite-1.0.2/vm/kite_compiler.c <==
/*****************************************************************************
 * Copyright (c) 2007, Mooneer Salem
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Kite Language organization nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MOONEER SALEM ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MOONEER SALEM BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "objs/kite_object.h"
#include "kite_vm.h"

typedef void* yyscan_t;
int yylex_init(yyscan_t*);
void yyset_extra(void *, yyscan_t);
void yyset_in(FILE *, yyscan_t);
int yylex_destroy(yyscan_t);
int yyparse (void *);

/*****************************************************************************
 * Compile Kite code from a file.
 ****************************************************************************/
kite_object_t *kite_vm_compile_from_file(kite_thread_t *thd, char *file)
{
    FILE *fp;
    kite_object_t *ret;
    
    fp = fopen(file, "r");
    if (fp == NULL) {
        return NULL;
    }

    ret = kite_vm_compile_from_fp(thd, fp, file, kite_new_class(
        thd, 
        kite_dereference_and_load(thd, "System.object"),
        ""
    ));
    fclose(fp);
    return ret;
}

/*****************************************************************************
 * Compile Kite code from a file.
 ****************************************************************************/
kite_object_t *
kite_vm_compile_from_file_without_create(kite_thread_t *thd,
                                         char *file, 
                                         kite_object_t *obj)
{
    FILE *fp;
    kite_object_t *ret;
    
    fp = fopen(file, "r");
    if (fp == NULL) {
        return NULL;
    }

    ret = kite_vm_compile_from_fp(thd, fp, file, obj);
    fclose(fp);
    return ret;
}

/*****************************************************************************
 * Compile Kite code from an open file.
 ****************************************************************************/
kite_object_t *kite_vm_compile_from_fp(kite_thread_t *thd, FILE *fp, char *f, kite_object_t *obj) 
{
    yyscan_t scanner;
    kite_compiler_t *compiler;
    kite_object_t *ret;
    
#ifndef HAVE_GC_H
    compiler = (kite_compiler_t*)calloc(sizeof(kite_compiler_t), 1);
#else
    compiler = (kite_compiler_t*)GC_malloc(sizeof(kite_compiler_t));
#endif /* HAVE_GC_H */
    if (compiler == NULL) {
        return NULL;
    }
    
    compiler->thd = thd;
    compiler->currentLine = 1;
    compiler->file = f;
    kite_push_stack(
        &compiler->obj_created, 
        FALSE, 
        obj
    );
    
    yylex_init(&scanner);
	yyset_extra(compiler, scanner);
    yyset_in(fp, scanner);
    yyparse(scanner);
	yylex_destroy(scanner);
	
    ret = kite_pop_stack(&compiler->obj_created);
    kite_destroy_stack(&compiler->obj_created);
    kite_destroy_stack(&compiler->decide_exits);
    ret->builtin_data.funcvalue.functype = FUNC_BYTECODE;
#ifndef HAVE_GC_H
    free(compiler);
#endif /* HAVE_GC_H */
    return ret;
}

typedef void* YY_BUFFER_STATE;
void yy_switch_to_buffer ( YY_BUFFER_STATE, yyscan_t );
void yy_delete_buffer ( YY_BUFFER_STATE, yyscan_t );
YY_BUFFER_STATE yy_scan_string ( const char *, yyscan_t );

/*****************************************************************************
 * Compile Kite code from a string.
 ****************************************************************************/
kite_object_t *kite_vm_compile_from_string(kite_thread_t *thd, char *code, int ln)
{
    kite_object_t *ret;
    yyscan_t scanner;
    kite_compiler_t *compiler;
    YY_BUFFER_STATE bufstate = NULL;

#ifndef HAVE_GC_H
    compiler = (kite_compiler_t*)calloc(sizeof(kite_compiler_t), 1);
#else
    compiler = (kite_compiler_t*)GC_malloc(sizeof(kite_compiler_t));
#endif /* HAVE_GC_H */
    if (compiler == NULL) {
        return NULL;
    }

    compiler->thd = thd;
    compiler->currentLine = ln;
    compiler->file = "?";
    kite_push_stack(
        &compiler->obj_created, 
        FALSE, 
        kite_new_class(
            thd, 
            kite_dereference_and_load(thd, "System.object"),
            ""
        )
    );

    yylex_init(&scanner);
	yyset_extra(compiler, scanner);
    bufstate = yy_scan_string(code, scanner);
    yy_switch_to_buffer(bufstate, scanner);
    yyparse(scanner);
    yy_delete_buffer(bufstate, scanner);
	yylex_destroy(scanner);

    ret = kite_pop_stack(&compiler->obj_created);
    ret->builtin_data.funcvalue.functype = FUNC_BYTECODE;
#ifndef HAVE_GC_H
    free(compiler);
#endif /* HAVE_GC_H */
    return ret;
}
